<?php
/* ouvrage
 * Copyright (C) 2017       Inovea-conseil.com     <info@inovea-conseil.com>
 */

/**
 * \file    class/actions_ouvrage.class.php
 * \ingroup ouvrage
 * \brief   Actionsouvrage
 *
 * ouvrage actions
 */

require_once DOL_DOCUMENT_ROOT . '/core/lib/functions.lib.php';
dol_include_once('/ouvrage/class/ouvrage.class.php');    // InfraS add

/**
 * Class ActionsSmsCommande
 */
class ActionsOuvrage
{
	/**
	 * @var DoliDB Database handler
	 */
	private $db;

	/**
	 * @var dash
	 */
	private $dash;

	/**
	 * @var Special Code
	 */
    private $special_code = 432406;	// InfraS changes 501028 to 432406 (we use the module number for better compatibility with other modules like subtotal)

	/**
	 *  Constructor
	 *
	 * @param DoliDB $db Database handler
	 */
	public function __construct($db)
	{
		global $conf;
		$this->dash = 0;//$conf->global->MAIN_PDF_DASH_BETWEEN_LINES;
		$this->db = $db;
	}

	public function printOriginObjectLine($parameters, &$object, &$action, $hookmanager)
	{
		global $restrictlist, $selectedLines;

		$line = &$parameters['line'];
		$contexts = explode(':', $parameters['context']);
		if (in_array('ordercard', $contexts) || in_array('invoicecard', $contexts)) {
			if (Ouvrage::isOuvrage($line)) {
				$object->printOriginLine($line, '', $restrictlist, '/core/tpl', $selectedLines);
				if (!empty($object->lines)) {
					foreach ($object->lines as $childline) {
						if ($childline->fk_parent_line == $line->id) {
							$object->printOriginLine($childline, '', $restrictlist, '/core/tpl', $selectedLines);
						}
					}
				}    // if (!empty($object->lines))

				return 1;
			}    // if (Ouvrage::isOuvrage($line))
		}    // if (in_array('ordercard',$contexts) || in_array('invoicecard',$contexts))

		return 0;
	}    // public_function printOriginObjectLine($parameters, &$object, &$action, $hookmanager)

	function formAddObjectLine($parameters = false)
	{
		global $conf, $langs, $user;
		global $object, $bcnd, $var;

		$maxDecimalsTotal = $conf->global->MAIN_MAX_DECIMALS_TOT;

		$usemargins = 0;
		$colspanbutton = 4;

		if (!empty($conf->global->MAIN_VIEW_LINE_NUMBER)) {
			$colspantitle = 10;
			$colspanlabel = 7;
			$colspandesc = 7;
		} else {
			$colspantitle = 9;
			$colspanlabel = 5;
			$colspandesc = 5;
		}

		if (!empty($conf->margin->enabled) && !empty($GLOBALS['object']->element) && in_array($GLOBALS['object']->element,
				array('facture', 'propal', 'askpricesupplier', 'commande'))) {
			$usemargins = 1;
			$colspantitle += 3;
			$colspanlabel++;
			$colspandesc += 1;
		}

		if (!empty($usemargins) && !empty($conf->global->DISPLAY_MARGIN_RATES) && $user->rights->margins->liretous) {
			$colspantitle += 2;
			$colspanlabel++;
			$colspandesc += 1;
		}

		if (!empty($usemargins) && !empty($conf->global->DISPLAY_MARK_RATES) && $user->rights->margins->liretous) {
			$colspantitle += 2;
			$colspanlabel++;
			$colspandesc += 1;
		}

		dol_include_once('/ouvrage/class/html.form_ouvrage.class.php');
		$form = new FormOuvrage($this->db);

		echo '<tr class="liste_titre nodrag nodrop"><td colspan="' . $colspantitle . '">' . $langs->trans($conf->global->OUVRAGE_TYPE . 'ADDOUVRAGE') . '</td></tr>';
		echo '<tr class="nodrag nodrop"><td colspan="' . $colspantitle . '">' . $form->select_ouvrage();

		echo '<div id="modal-ouvrage" title="' . $langs->trans($conf->global->OUVRAGE_TYPE . "OUVRAGE") . '" style="display:none;" ></div>';

		echo '<script>';
		$btnSend = $langs->trans("Add");
		$url = dol_buildpath('/ouvrage/ajax/ajaxOuvrageForm.php', 1);
		$select2selctingMethod = (int) DOL_VERSION >= '7' ? 'select2:selecting' : 'select2-selecting';
		$choiceIdParam = (int) DOL_VERSION >= '7' ? 'e.params.args.data.id' : 'e.choice.id';
		$sobject['id'] = $object->id;
		$sobject['element'] = $object->element;
		$sobject2 = json_encode($sobject);
		$variantsEnabled = ($conf->variants->enabled) ? 1 : -1;
		echo <<<HEREDOC
        $(document).on("$select2selctingMethod", '#ouvrageid', function (e) {
        var ouvrageid = $choiceIdParam;
        var o = $sobject2;
        $.ajax({
            url: "$url",
            data: {id: ouvrageid, o: o},
            dataType: 'html',
            type: 'POST'
        }).done(function( data ) {
            $( "#modal-ouvrage" ).html(data);
                $( "#modal-ouvrage" ).dialog({
               resizable: true,
               height: "auto",
               width: 1400,	// InfraS change
               modal: true,
                   buttons:
               [{
                 text: "$btnSend",
                 click: function () {
                    if ($('#tablelinesouvrage input[name="ouvrage[qty]"]').val() > 0) {
                        if ($variantsEnabled) {
                            $('[id^="variants_options_product"]').each(function () {
                                $(this).empty();
                            });
                        }
                        calculTotalOuvrage();
                        $('#tablelinesouvrage').find('input, select').each(function(){
                            $('#addproduct').prepend('<input type="hidden" name="'+$(this).attr('name')+'" value="'+$(this).val()+'" />');
                        });
                        $('#addproduct').prepend('<input type="hidden" name="ouvrage[price]" value="'+$('#tablelinesouvrage').find('td.ouvrage-price-unit').text()+'" />');
                        $('#prod_entry_mode_predef').prop('checked', 'checked');
                        $('#addproduct').submit();
                    }
                 },

             }],
            });
HEREDOC;

		if (isset($conf->fournisseur) && $conf->fournisseur->enabled) {
			$urlAjaxFourn = dol_buildpath('/fourn/ajax/getSupplierPrices.php?bestpricefirst=1', 1);
			$langs->load('stocks');
			$langInputPrice = $langs->trans("InputPrice");

			// Margin Configuration
			$marginConfig = '';
			if (isset($conf->global->MARGIN_TYPE)) {
				if ($conf->global->MARGIN_TYPE == '1') {
					$marginConfig = 'bestsupplierprice';
				}
				if ($conf->global->MARGIN_TYPE == 'pmp') {
					$marginConfig = 'pmp';
				}
				if ($conf->global->MARGIN_TYPE == 'costprice') {
					$marginConfig = 'costprice';
				}
			}

			echo <<<HEREDOC
            $('#tablelinesouvrage input[name$="[marge]"]').each(function(){
                var productId = $(this).data('product');
                var inputMarge = this;

                var idSelected = 'fournprice_predef_'+productId;

                if (data && data.length > 0)
    	    	{
                    var options = '';
                    var defaultkey = '';
                    var defaultprice = '';
                    var bestpricefound = 0;

                    var bestpriceid = 0; var bestpricevalue = 0;
                    var pmppriceid = 0; var pmppricevalue = 0;
                    var costpriceid = 0; var costpricevalue = 0;

                    /* setup of margin calculation */
                    var defaultbuyprice = '$marginConfig';
                    $.post('$urlAjaxFourn', { 'idprod': productId }, function(data) {
                        if (data && data.length > 0) {
                            var i = 0;
    	      		$(data).each(function() {
    	      			if (this.id != 'pmpprice' && this.id != 'costprice')
    		      		{
    		        		i++;
                            this.price = parseFloat(this.price); // to fix when this.price >0
    			      		// If margin is calculated on best supplier price, we set it by defaut (but only if value is not 0)
    		      			if (bestpricefound == 0 && this.price > 0) { defaultkey = this.id; defaultprice = this.price; bestpriceid = this.id; bestpricevalue = this.price; bestpricefound=1; }	// bestpricefound is used to take the first price > 0
    		      		}
    	      			if (this.id == 'pmpprice')
    	      			{
    	      				// If margin is calculated on PMP, we set it by defaut (but only if value is not 0)
    			      		if ('pmp' == defaultbuyprice || 'costprice' == defaultbuyprice)
    			      		{
    			      			if (this.price > 0) {
    				      			defaultkey = this.id; defaultprice = this.price; pmppriceid = this.id; pmppricevalue = this.price;
    			      			}
    			      		}
    	      			}
    	      			if (this.id == 'costprice')
    	      			{
    	      				// If margin is calculated on Cost price, we set it by defaut (but only if value is not 0)
    			      		if ('costprice' == defaultbuyprice)
    			      		{
    		      				if (this.price > 0) { defaultkey = this.id; defaultprice = this.price; costpriceid = this.id; costpricevalue = this.price; }
    		      				else if (pmppricevalue > 0) { defaultkey = pmppriceid; defaultprice = pmppricevalue; }
    			      		}
    	      			}
    	        		options += '<option value="'+this.id+'" price="'+this.price+'">'+this.label+'</option>';
    	      		});
    	      		options += '<option value="inputprice" price="'+defaultprice+'">$langInputPrice</option>';

                        var selectedElt = $("<select></select>").attr('id', idSelected).html(options);

    	      		$(inputMarge).before(selectedElt);
                        $(selectedElt).after('<br/>')
    	      		if (defaultkey != '')
    				{
    		      		$(selectedElt).val(defaultkey);
    		      	}

    	      		/* At loading, no product are yet selected, so we hide field of buying_price */
    	      		$(inputMarge).hide();

    	      		/* Define default price at loading */
    	      		var defaultprice = $(selectedElt).find('option:selected').attr("price");
    			    $(inputMarge).val(defaultprice);

    	      		$(selectedElt).change(function() {
    	      			/* Hide field buying_price according to choice into list (if 'inputprice' or not) */
    					var linevalue=$(this).find('option:selected').val();
    	        		var pricevalue = $(this).find('option:selected').attr("price");
    	        		if (linevalue != 'inputprice' && linevalue != 'pmpprice') {
    	          			$(inputMarge).val(pricevalue).hide();	/* We set value then hide field */
    	        		}
    	        		if (linevalue == 'inputprice') {
    		          		$(inputMarge).show();
    	        		}
    	        		if (linevalue == 'pmpprice') {
    	        			$(inputMarge).val(pricevalue);
    		          		$(inputMarge).hide();
    	        		}

                    $(inputMarge).trigger('change');
    				});
                        }
                    },
                    'json');
                    }});
HEREDOC;
		}

		echo <<<HEREDOC
        setTimeout(calculTotalOuvrage, 500);
        //setInterval(function(){calculTotalOuvrage();}, 100);
        //setInterval(calculTotalOuvrage, 100);
        });

});

   $(document).on('change', '#tva_tx_ouvrage', function(e){
       $('#tablelinesouvrage td.product-tva').html($(this).val());
   });
   $(document).on('keyup change', '#tablelinesouvrage input[name="ouvrage[qty]"]', function(e){
        var qtyOuvrage = $(this).val();
        $('#tablelinesouvrage td.product-qty input').each(function(){
            $(this).val(qtyOuvrage*$(this).data('default'));
            $(this).trigger('change');
        });
        e.stopPropagation();
   });
    var reducOuvrage = 0;
   $(document).on('keyup change', '#tablelinesouvrage input[name="ouvrage[reduc]"]', function(e){
        var thisval = parseFloat($(this).val());
        thisval = isNaN(thisval) ? 0 : thisval;
        var tmpReducOuvrage = thisval - reducOuvrage;
        reducOuvrage =  thisval;
        $('#tablelinesouvrage td.product-reduc input').each(function(){
            //var thisval = parseFloat($(this).val());
            //thisval = isNaN(thisval) ? 0 : thisval;
            //var reducProduct = thisval+tmpReducOuvrage
            $(this).val(reducOuvrage);
            $(this).trigger('change');
        }).promise().done( function(){
            calculTotalOuvrage();
        } );
        e.stopPropagation();
   });

   var useCostPricePerUnit = 0;
   var useCheckboxes = 0;	// InfraS add

HEREDOC;

		if (!empty($conf->global->OUVRAGE_USE_COST_PRICE_PER_UNIT)) {
			echo <<<HEREDOC
   			useCostPricePerUnit = 1;
HEREDOC;
		}

		// InfraS add begin
		if (!empty($conf->global->OUVRAGE_USE_LINE_CHECKBOX)) {
			echo <<<HEREDOC
   useCheckboxes = 1;
   $(document).on('keyup change', '#tablelinesouvrage td.product-check input', function(e){
		if ($(this).is(":checked")) {
			$(this).parents('tr').find('td.product-check input[type="hidden"]').val("1");
		}
		else if($(this).is(":not(:checked)")) {
			$(this).parents('tr').find('td.product-check input[type="hidden"]').val("0");
		}
		calculTotalOuvrage();
		e.stopPropagation();
   });
HEREDOC;
		}
		// InfraS add end
		echo <<<HEREDOC
   $(document).on('keyup change', '#tablelinesouvrage td.product-qty input, #tablelinesouvrage td.product-price input, #tablelinesouvrage td.product-reduc input, #tablelinesouvrage td.product-marge input', function(e){
       var qtyProduct = $(this).parents('tr').find('td.product-qty input').val();

       // On change la quantité par défaut
       if ($('#tablelinesouvrage td.product-qty input').is(':focus') && $('#tablelinesouvrage input[name="ouvrage[qty]"]').val() == 1) {
                $(this).data('default', $(this).val());
       }

       var priceProduct = $(this).parents('tr').find('td.product-price input').val();
       var reducProduct = parseFloat($(this).parents('tr').find('td.product-reduc input').val());

       reducProduct = isNaN(reducProduct) ? 0 : reducProduct;

       $(this).parents('tr').find('td.product-reduc input').val(reducProduct);

       var totalProduct = priceProduct*qtyProduct*((100-reducProduct)/100);

       $(this).parents('tr').find('td.product-price-total').text(parseFloat(totalProduct).toFixed($maxDecimalsTotal));
        e.stopPropagation();
   });

   $(document).on('keyup change', '#tablelinesouvrage td.product-reduc input', function(e){
       if ($(this).val() != $('#tablelinesouvrage input[name="ouvrage[reduc]"]').val()) {
            calculTotalOuvrage();
            // On calcul la réduc de l'ouvrage
            var reducOuvrage = parseFloat($('#tablelinesouvrage td.ouvrage-price-total').text()) / $('#tablelinesouvrage input[name="ouvrage[qty]"]').val();
            reducOuvrage = reducOuvrage * 100 / parseFloat($('#tablelinesouvrage td.ouvrage-price-unit').text());
            reducOuvrage = 100 - reducOuvrage;
            $('#tablelinesouvrage input[name="ouvrage[reduc]"]').val(parseFloat(reducOuvrage).toFixed(2));
       }
   });


   $(document).on('keyup change', '#tablelinesouvrage td.product-qty input, #tablelinesouvrage td.product-price input, #tablelinesouvrage td.product-check input', function(e){
       calculTotalOuvrage();
       var puOuvrage = 0;

       $('#tablelinesouvrage tr').each(function(){
           // InfraS add begin
		   if (useCheckboxes == 1) {
			   var validProduct = $(this).find('td.product-check input[type="hidden"]').val();
			   if (validProduct == 1) {
					if ($(this).find('td.product-qty').length > 0 && $(this).find('td.product-qty input').val() > 0) {
						puOuvrage += $(this).find('td.product-qty input').val() * $(this).find('td.product-price input').val();
					}
			   }
		   }
		   else	{
			   // InfraS add end
				if ($(this).find('td.product-qty').length > 0 && $(this).find('td.product-qty input').val() > 0) {
					puOuvrage += $(this).find('td.product-qty input').val() * $(this).find('td.product-price input').val();
				}
		   }	// InfraS add
       }).promise().done( function(){
            puOuvrage = puOuvrage / $('#tablelinesouvrage input[name="ouvrage[qty]"]').val();
            puOuvrage = isNaN(puOuvrage) ? 0 : puOuvrage;
            $('#tablelinesouvrage td.ouvrage-price-unit').text(parseFloat(puOuvrage).toFixed($maxDecimalsTotal));
       });

   });

    $(document).on('focusout', '#tablelinesouvrage', function(e){
       calculTotalOuvrage();
   });


   function calculTotalOuvrage() {
       var totalOuvrage = 0;
        $('#tablelinesouvrage td.product-price-total').each(function(){
           // InfraS add begin
		   if (useCheckboxes == 1) {
			   var validProduct = $(this).parents('tr').find('td.product-check input[type="hidden"]').val();
			   if (validProduct == 1) {
				   totalOuvrage += parseFloat($(this).text().replace(',','.').replace(' ',''));
			   }
		   }
		   else	{
			   // InfraS add end
			   totalOuvrage += parseFloat($(this).text().replace(',','.').replace(' ',''));
		   }	// InfraS add
        }).promise().done( function(){
            totalOuvrage = parseFloat(totalOuvrage).toFixed($maxDecimalsTotal);
            $('#tablelinesouvrage td.ouvrage-price-total').text(totalOuvrage);
        } );


        // Prix de revient ouvrage
        var prixRevientOuvrage = 0;
        if ($('input[name="ouvrage[marge]"]').length > 0) {
            $('#tablelinesouvrage td.product-marge input').each(function(){
                var qtyProduct = $(this).parents('tr').find('td.product-qty input').val();
                var prixRevientProduct = 0;

				// InfraS add begin
			   if (useCheckboxes == 1) {
				   var validProduct = $(this).parents('tr').find('td.product-check input[type="hidden"]').val();
				   if (validProduct == 1) {
						if ($(this).val() == 0 && $(this).parents('td').find('select option:selected').attr('price') != '' && $(this).parents('td').find('select option:selected').attr('price') != 'undefined') {
							prixRevientOuvrage += parseFloat($(this).parents('td').find('select option:selected').attr('price'))*qtyProduct;
						} else {
							if ($(this).val().replace(',','.').length > 0) {
								prixRevientOuvrage += parseFloat($(this).val().replace(',','.').replace(' ',''))*qtyProduct;
							} else {
								prixRevientOuvrage += 0;
							}
						}
				   }
			   }
			   else	{
				   // InfraS add end
					if ($(this).val() == 0 && $(this).parents('td').find('select option:selected').attr('price') != '' && $(this).parents('td').find('select option:selected').attr('price') != 'undefined') {
						prixRevientOuvrage += parseFloat($(this).parents('td').find('select option:selected').attr('price'))*qtyProduct;
					} else {
						if ($(this).val().replace(',','.').length > 0) {
							prixRevientOuvrage += parseFloat($(this).val().replace(',','.').replace(' ',''))*qtyProduct;
						} else {
							prixRevientOuvrage += 0;
						}
					}
			   }	// InfraS add
            });

            prixRevientOuvrage = isNaN(prixRevientOuvrage) ? 0 : prixRevientOuvrage;
            if (useCostPricePerUnit == 1) {
            	var qtyOuvrage = $('#tablelinesouvrage input[name="ouvrage[qty]"]').val();
            	if (qtyOuvrage != 0) {
            		prixRevientOuvrage = prixRevientOuvrage / qtyOuvrage
            	}
            }
            $('input[name="ouvrage[marge]"]').val(parseFloat(prixRevientOuvrage).toFixed($maxDecimalsTotal));
        }
   }

HEREDOC;

		echo '</script>';

		echo '</td></tr>';
	}
	function createFrom($parameters, &$object, $action, $hookmanager) {
		global $user,$db;

		$object->fetch_lines();
		$fk_parent_line = null;

		$lines = $object->lines;

		$columns = array_column($lines, 'rowid');
		array_multisort($columns, SORT_ASC, $lines);


		//print '<pre>'.print_r($lines,1).'</pre>';
		//print '<pre>'.print_r($object->lines ,1).'</pre>';

		$i=0;
		foreach ( $lines as $line) {

			if (empty($line->remise)) {
				$line->remise = 0;
			}
			$line->rang = $i;
			if (!empty($line->fk_parent_line)) {
				$line->fk_parent_line = $fk_parent_line;

			}
			if ($line->product_type == 9) {
				$fk_parent_line = $line->rowid;
			}
			$line->update($user);
			$i++;
		}

	//	print '<pre>'.print_r( $object->lines,1).'</pre>';
		//print '<pre>'.print_r($object->lines,1).'</pre>';

	//	print '<pre>'.print_r($object->lines,1).'</pre>';

	}

	/**
	 *    Return action of hook
	 *
	 * @param object            Linked object
	 */
	function doActions($parameters = false, &$object, &$action = '')
	{
		global $conf, $user, $langs;

		$langs->load("ouvrage@ouvrage");
		dol_include_once('/ouvrage/class/ouvrage.class.php');
		dol_include_once('/product/class/product.class.php');
		if (GETPOST('ouvrage') && $action == 'addline') {
			$error = 0;
			$ouvrage_post = GETPOST('ouvrage');



			// Open DSI -- Begin
			// ksort($ouvrage_post['product']);	// InfraS change
			// Open DSI -- End
			$products = $ouvrage_post['product'];
			$newProducts = $ouvrage_post['newProduct'];

			//echo "<pre>".print_r($ouvrage_post,1)."</pre>";
			$tot = 0;
			if (!empty($products)) {
				foreach ($products as $product) {
					$tot += !empty($conf->global->OUVRAGE_USE_LINE_CHECKBOX) ? $product['check'] : $product['qty'];	// InfraS change
				}
			}

			if (!empty($newProducts)) {
				foreach ($newProducts as $product) {
					$tot += !empty($conf->global->OUVRAGE_USE_LINE_CHECKBOX) ? $product['check'] : $product['qty'];	// InfraS change
				}
			}

			if (!empty($products)) {
				uasort($products, function ($a, $b) {
					return $a['order'] - $b['order'];
				});    // InfraS add
			}

			if ($tot > 0) {
				$ouvrage = new Ouvrage($this->db);
				$ouvrage->fetch($ouvrage_post['id']);

				// Clean parameters
				$label = trim($ouvrage->label);
				if (!empty(trim($ouvrage_post['desc']))) {
					$description = trim($ouvrage_post['desc']);
				} else {
					$description = trim($ouvrage->description) != '' ? trim($ouvrage->description) : "";
				}
				$product_type = 9;
				$special_code = $this->special_code;
				$ouvrage_qty = $ouvrage_post['qty'];
				$ouvrage_fk_product = $ouvrage_post['fk_product'];
				$ouvrage_price = $ouvrage_post['price']; /*/ $ouvrage_post['qty']*/
				//$ouvrage_price = 100 * $ouvrage_price / 100 - $ouvrage_post['reduc'];
				$ouvrage_unit = $ouvrage_post['unit'];
				$ouvrage_remise = $ouvrage_post['reduc'];
				$ouvrage_buy_price = isset($ouvrage_post['marge']) ? $ouvrage_post['marge'] : 0;
				$tva = GETPOST('tva_tx_ouvrage', 'alpha');
				$rangtouse = $object->line_max() + 1;
				$situation = "";
				// TODO uniformiser
				//              addline($desc, $pu_ht, $qty, $txtva, $txlocaltax1 = 0.0, $txlocaltax2 = 0.0, $fk_product = 0, $remise_percent = 0.0, $price_base_type = 'HT', $pu_ttc = 0.0, $info_bits = 0, $type = 0, $rang = -1, $special_code = 0, $fk_parent_line = 0, $fk_fournprice = 0, $pa_ht = 0, $label = '', $date_start = '', $date_end = '', $array_options = 0, $fk_unit = null, $origin = '', $origin_id = 0, $pu_ht_devise = 0, $fk_remise_except = 0)

				if ($object->element == 'propal') {
					$fields = array(
						$description,
						$ouvrage_price,
						$ouvrage_qty,
						$tva,
						0,
						0,
						$ouvrage_fk_product,
						$ouvrage_remise,
						"HT",
						0,
						0,
						$product_type,
						$rangtouse,
						$special_code,
						0,
						0,
						$ouvrage_buy_price,
						$label,
						'',
						'',
						0,
						$ouvrage_unit
					);
				}
				if ($object->element == 'commande') {
					$fields = array(
						$description,
						$ouvrage_price,
						$ouvrage_qty,
						$tva,
						0,
						0,
						$ouvrage_fk_product,
						$ouvrage_remise,
						0,
						0,
						'HT',
						0,
						null,
						null,
						$product_type,
						$rangtouse,
						$special_code,
						0,
						null,
						$ouvrage_buy_price,
						$label,
						'',
						'',
						0,
						$ouvrage_unit
					);
				}
				//    public function addline($desc, $pu_ht, $qty, $txtva, $txlocaltax1 = 0, $txlocaltax2 = 0, $fk_product = 0, $remise_percent = 0, $date_start = '', $date_end = '', $ventil = 0, $info_bits = 0, $fk_remise_except = '', $price_base_type = 'HT', $pu_ttc = 0, $type = self::TYPE_STANDARD, $rang = -1, $special_code = 0, $origin = '', $origin_id = 0, $fk_parent_line = 0, $fk_fournprice = null, $pa_ht = 0, $label = '', $array_options = 0, $situation_percent = 100, $fk_prev_id = 0, $fk_unit = null, $pu_ht_devise = 0)
				if ($object->element == 'facture') {
					$fields = array(
						$description,
						$ouvrage_price,
						$ouvrage_qty,
						$tva,
						0,
						0,
						$ouvrage_fk_product,
						$ouvrage_remise,
						null,
						null,
						0,
						0,
						0,
						'HT',
						0,
						$product_type,
						$rangtouse,
						$special_code,
						'',
						0,
						0,
						null,
						$ouvrage_buy_price,
						$label,
						0,
						$situation,
						0,
						$ouvrage_unit
					);
				}

				$result_ouvrage = $object->addline($fields[0], $fields[1], $fields[2], $fields[3], $fields[4],
					$fields[5], $fields[6], $fields[7], $fields[8], $fields[9], $fields[10], $fields[11], $fields[12],
					$fields[13], $fields[14], $fields[15], $fields[16], $fields[17], $fields[18], $fields[19],
					$fields[20], $fields[21], $fields[22], $fields[23], $fields[24], $fields[25], $fields[26],
					$fields[27], $fields[28]);

				//          $rangproduit = $rangtouse + 1;

				foreach ($products as $product_id => $p) {
					$p = $products[$product_id];
					if ((!empty($conf->global->OUVRAGE_USE_LINE_CHECKBOX) && $p['check'] == 1) || (empty($conf->global->OUVRAGE_USE_LINE_CHECKBOX) && $p['qty'] > 0)) {	// InfraS change
						$product = new Product($this->db);
						$product->fetch($product_id);

						if (!empty($conf->global->OUVRAGE_USE_LINE_VAT)) {
							$tva_product = $p['tva'];
							if ($tva_product != '') {
								$tva = $tva_product;
							} else {
								$tva = $product->tva_tx;
							}
						}

						$label = trim($product->label);
						$description = trim($product->description);
						$product_type = $product->type;
						$product_price = $p['price_ht'];
						$product_remise = $p['reduc'];
						$product_buy_price = isset($p['marge']) ? $p['marge'] : 0;
						$qty = $p['qty'];
						$rangproduit = $rangtouse + $p['order'];    // InfraS add
						$special_code = 0;
						$unit = $product->fk_unit;
						if ($p['unit'] != "none") {
							$unit = $p['unit'];
						}
						//$result_ouvrage = 0;


						// TODO uniformiser
						if ($object->element == 'propal') {
							$fields = array(
								$description,
								$product_price,
								$qty,
								$tva,
								0,
								0,
								$product_id,
								$product_remise,
								"HT",
								0,
								0,
								$product_type,
								$rangproduit,
								$special_code,
								$result_ouvrage,
								0,
								$product_buy_price,
								$label,
								'',
								'',
								0,
								$unit
							);
						}
						if ($object->element == 'commande') {
							$fields = array(
								$description,
								$product_price,
								$qty,
								$tva,
								0,
								0,
								$product_id,
								$product_remise,
								0,
								0,
								'HT',
								0,
								null,
								null,
								$product_type,
								$rangproduit,
								$special_code,
								$result_ouvrage,
								null,
								$product_buy_price,
								$label,
								'',
								'',
								0,
								$unit
							);
						}
						if ($object->element == 'facture') {
							$fields = array(
								$description,
								$product_price,
								$qty,
								$tva,
								0,
								0,
								$product_id,
								$product_remise,
								null,
								null,
								0,
								0,
								'',
								'HT',
								0,
								$product_type,
								$rangproduit,
								$special_code,
								'',
								0,
								$result_ouvrage,
								0,
								$product_buy_price,
								$label,
								0,
								$situation,
								0,
								$unit
							);
						}

						//  public function addline($desc, $pu_ht, $qty, $txtva, $txlocaltax1 = 0, $txlocaltax2 = 0, $fk_product = 0, $remise_percent = 0, $date_start = '', $date_end = '', $ventil = 0, $info_bits = 0, $fk_remise_except = '', $price_base_type = 'HT', $pu_ttc = 0, $type = self::TYPE_STANDARD, $rang = -1, $special_code = 0, $origin = '', $origin_id = 0, $fk_parent_line = 0, $fk_fournprice = null, $pa_ht = 0, $label = '', $array_options = 0, $situation_percent = 100, $fk_prev_id = 0, $fk_unit = null, $pu_ht_devise = 0)
						$result = $object->addline($fields[0], $fields[1], $fields[2], $fields[3], $fields[4],
							$fields[5], $fields[6], $fields[7], $fields[8], $fields[9], $fields[10],
							$fields[11], $fields[12], $fields[13], $fields[14], $fields[15], $fields[16],
							$fields[17], $fields[18], $fields[19], $fields[20], $fields[21], $fields[22],
							$fields[23], $fields[24], $fields[25], $fields[26], $fields[27], $fields[28]);
					}
					// }
				}

				$rangtouse += count($products);
				//echo "<pre>".print_r($newProducts,1)."</pre>";
				if (!empty($newProducts)) {
					foreach ($newProducts as $p) {
						if ((!empty($conf->global->OUVRAGE_USE_LINE_CHECKBOX) && $p['check'] == 1) || (empty($conf->global->OUVRAGE_USE_LINE_CHECKBOX) && $p['qty'] > 0)) {	// InfraS change
							if (!empty($p['productId'])) {
								$product = new Product($this->db);
								$product->fetch($p['productId']);

								if (!empty($conf->global->OUVRAGE_USE_LINE_VAT)) {
									$tva_product = $p['tva'];
									if ($tva_product != '') {
										$tva = $tva_product;
									} else {
										$tva = $product->tva_tx;
									}
								}

								$label = trim($product->label);
								$description = trim($product->description);
								$product_type = $product->type;
								$product_price = $p['price_ht'];
								$product_remise = $p['reduc'];
								$product_buy_price = isset($p['buyprice']) ? $p['buyprice'] : 0;	// InfraS change
								$qty = $p['qty'];
								$rangproduit = $rangtouse + $p['order'];    // InfraS add
								$special_code = 0;
								$unit = $product->fk_unit;
								if ($p['unit'] != "none") {
									$unit = $p['unit'];
								}
							} else {
								if (!empty($conf->global->OUVRAGE_USE_LINE_VAT)) {
									$tva = $p['tva'];
								}

								$label = trim($p['label']);	// InfraS change
								$description = trim($p['description']);	// InfraS change
								$product_type = $p['productType'];
								$product_price = $p['price_ht'];
								$product_remise = $p['reduc'];
								$product_buy_price = isset($p['buyprice']) ? $p['buyprice'] : 0;
								$qty = $p['qty'];
								$rangproduit = ++$rangtouse;    // InfraS add
								$special_code = 0;
								$unit = "";
								if ($p['unit'] != "none") {
									$unit = $p['unit'];
								}
							}

							//$result_ouvrage = 0;


							// TODO uniformiser
							if ($object->element == 'propal') {
								$fields = array(
									$description,
									$product_price,
									$qty,
									$tva,
									0,
									0,
									$p['productId'],
									$product_remise,
									"HT",
									0,
									0,
									$product_type,
									$rangproduit,
									$special_code,
									$result_ouvrage,
									0,
									$product_buy_price,
									$label,
									'',
									'',
									0,
									$unit
								);
							}
							if ($object->element == 'commande') {
								$fields = array(
									$description,
									$product_price,
									$qty,
									$tva,
									0,
									0,
									$p['productId'],
									$product_remise,
									0,
									0,
									'HT',
									0,
									null,
									null,
									$product_type,
									$rangproduit,
									$special_code,
									$result_ouvrage,
									null,
									$product_buy_price,
									$label,
									'',
									'',
									0,
									$unit
								);
							}
							if ($object->element == 'facture') {
								$fields = array(
									$description,
									$product_price,
									$qty,
									$tva,
									0,
									0,
									$p['productId'],
									$product_remise,
									null,
									null,
									0,
									0,
									'',
									'HT',
									0,
									$product_type,
									$rangproduit,
									$special_code,
									'',
									0,
									$result_ouvrage,
									0,
									$product_buy_price,
									$label,
									0,
									$situation,
									0,
									$unit
								);
							}

							//  public function addline($desc, $pu_ht, $qty, $txtva, $txlocaltax1 = 0, $txlocaltax2 = 0, $fk_product = 0, $remise_percent = 0, $date_start = '', $date_end = '', $ventil = 0, $info_bits = 0, $fk_remise_except = '', $price_base_type = 'HT', $pu_ttc = 0, $type = self::TYPE_STANDARD, $rang = -1, $special_code = 0, $origin = '', $origin_id = 0, $fk_parent_line = 0, $fk_fournprice = null, $pa_ht = 0, $label = '', $array_options = 0, $situation_percent = 100, $fk_prev_id = 0, $fk_unit = null, $pu_ht_devise = 0)
							$result = $object->addline($fields[0], $fields[1], $fields[2], $fields[3], $fields[4],
								$fields[5], $fields[6], $fields[7], $fields[8], $fields[9], $fields[10],
								$fields[11], $fields[12], $fields[13], $fields[14], $fields[15], $fields[16],
								$fields[17], $fields[18], $fields[19], $fields[20], $fields[21], $fields[22],
								$fields[23], $fields[24], $fields[25], $fields[26], $fields[27], $fields[28]);
						}	// InfraS add
					}

				}

				$object->update_price(1);
				;
				if ($object->element == 'facture') {
					Header('Location: ' . $_SERVER["PHP_SELF"] . '?facid=' . $object->id);
				} else {
					Header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
				}
			}
		}

		/**
		 * Suppression Line
		 */
		if ($action == 'confirm_deleteline' && GETPOST('confirm') == 'yes') {
			$error = 0;
			$lineid = GETPOST('lineid', 'int');
			//$this->db->begin();

			//var_dump($lineid);exit;

			// on recherche la ligne à supprimer
			foreach ($object->lines as $line) {
				if ($line->rowid == $lineid) {
					$lineToDelete = $line;
				}
			}

			if ($lineToDelete->special_code == $this->special_code) {
				// Call trigger
				include_once DOL_DOCUMENT_ROOT . '/core/class/interfaces.class.php';
				$interface = new Interfaces($this->db);
				$result = $interface->run_triggers('OUVRAGE_DELETE', $lineToDelete, $user, $langs, $conf);
				if ($result < 0) {
					$error++;
				}
				// End call triggers
			}
		}

		/**
		 * Mise à jour
		 */
		if (($action == 'updateligne' || $action == 'updateline') && GETPOST('ouvrage_update') == 1) {
			$lineid = GETPOST('lineid', 'int');

			// on recherche la ligne modifiée
			foreach ($object->lines as $line) {
				if ($line->rowid == $lineid) {
					$lineUpdate = $line;
				}
			}

			// extrafields
			if ($lineUpdate != null) {
				require_once DOL_DOCUMENT_ROOT . '/core/class/extrafields.class.php';
				$extrafieldsline = new ExtraFields($this->db);
				$extralabelsline = $extrafieldsline->fetch_name_optionals_label($object->table_element_line);
				$array_options = $extrafieldsline->getOptionalsFromPost($extralabelsline);
				if (is_array($array_options) && count($array_options) > 0) {
					$lineUpdate->array_options = $array_options;
				}
			}

			include_once DOL_DOCUMENT_ROOT . '/core/class/interfaces.class.php';
			$interface = new Interfaces($this->db);
			$result = $interface->run_triggers('OUVRAGE_UPDATE', $lineUpdate, $user, $langs, $conf);
			if ($result < 0) {
				$error++;
			}
			unset($_POST['action']);
			return 1;
			if ($object->element == 'facture') {
				Header('Location: ' . $_SERVER["PHP_SELF"] . '?facid=' . $object->id);
			}

			Header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
		}

		if ($action == 'builddoc') {
			$_SESSION['OUVRAGE_HIDE_PRODUCT_DETAIL'] = GETPOST('OUVRAGE_HIDE_PRODUCT_DETAIL');
			$_SESSION['OUVRAGE_HIDE_PRODUCT_DESCRIPTION'] = GETPOST('OUVRAGE_HIDE_PRODUCT_DESCRIPTION');
            $_SESSION['OUVRAGE_SHOW_PRODUCT_QTY'] = GETPOST('OUVRAGE_SHOW_PRODUCT_QTY');
			$_SESSION['OUVRAGE_HIDE_MONTANT'] = GETPOST('OUVRAGE_HIDE_MONTANT');
		}
	}

	function printObjectLine($parameters, &$object, &$action, $hookmanager)    // InfraS change
	{
		global $conf, $langs, $user;    // InfraS change
		global $form, $bc, $bcnd;

		$element = $object->element;
		if ($element == 'propal') {
			$element = 'propale';
		}

		$lineid = GETPOST('lineid', 'int');
		$lineToUpdate = &$parameters['line'];    // InfraS add

		if ($action == 'editline') {
			/*		// InfraS change begin
            //On recherche la ligne à modifier
            foreach ($object->lines as $line) {
                if ($line->rowid == $lineid) {
                    $lineToUpdate = $line;
                }
            }
			*/        // InfraS change end
			if ($lineToUpdate->special_code == $this->special_code && GETPOST('lineid', 'int') == $lineToUpdate->id) {    // InfraS change
				echo '<input type="hidden" name="special_code" value="' . $this->special_code . '" />';
				echo '<input type="hidden" name="label" value="' . $lineToUpdate->label . '"/>';
				echo '<input type="hidden" name="ouvrage_update" value="1" />';

				$textHelp = $form->textwithpicto('',
					$langs->trans($conf->global->OUVRAGE_TYPE . "OUVRAGE_IMPOSSIBLEUPDATE"));
				$textHelp = preg_replace('/\'/', '\\\'', $textHelp);

				print "<script>
                    $(function() {
                        setTimeout(function(){
                            $('#price_ht').prop('disabled', true);
                            $('#buying_price').prop('disabled', true);
                            $('#price_ht').attr('size', '4');
                            if (! ($('#price_ht').next().val() !== undefined)) {
                                $('#price_ht').after('" . $textHelp . "');
                            }
                            if (". (!empty($conf->global->OUVRAGE_USE_LINE_VAT)?'true':'false') .") {
                            	$('#tva_tx').prop('disabled', true);
                            }
                            $('#product_desc').before('".img_object('', 'product', 'class = "paddingright classfortooltip"', 0, 0, 0).'&nbsp;<b>'.dol_escape_js($lineToUpdate->label)."</b>');	// InfraS change
                        }, 1);
                    });
                </script>";
			}

			if (!is_null($lineToUpdate->fk_parent_line) && $lineToUpdate->fk_parent_line > 0) {
				// On vérifie si la ligne supprimée appartient à un ouvrage
				foreach ($object->lines as $line) {
					if ($line->rowid == $lineToUpdate->fk_parent_line) {
						$parentLineToUpdate = $line;
					}
				}

				if ($parentLineToUpdate->special_code == $this->special_code) {
					echo '<input type="hidden" name="fk_parent_line" value="' . $lineToUpdate->fk_parent_line . '" />';
					echo '<input type="hidden" name="ouvrage_update" value="1" />';
				}
			}
		}
	}

	/**
	 *    Return HTML form for builddoc bloc
	 */
	function formBuilddocOptions($parameters = false)
	{
		global $conf, $langs;
		global $bc, $var;

		$langs->load('ouvrage@ouvrage');

		$out = '';


		$checkedHideDetails = (!isset($_SESSION['OUVRAGE_HIDE_PRODUCT_DETAIL']) && $conf->global->OUVRAGE_HIDE_PRODUCT_DETAIL == 1) || (isset($_SESSION['OUVRAGE_HIDE_PRODUCT_DETAIL']) && $_SESSION['OUVRAGE_HIDE_PRODUCT_DETAIL'] == 1) ? 'checked="checked"' : '';
		$checkedHideDesc = (!isset($_SESSION['OUVRAGE_HIDE_PRODUCT_DESCRIPTION']) && $conf->global->OUVRAGE_HIDE_PRODUCT_DESCRIPTION == 1) || (isset($_SESSION['OUVRAGE_HIDE_PRODUCT_DESCRIPTION']) && $_SESSION['OUVRAGE_HIDE_PRODUCT_DESCRIPTION'] == 1) ? 'checked="checked"' : '';
		$checkedShowProductQty = (!isset($_SESSION['OUVRAGE_SHOW_PRODUCT_QTY']) && $conf->global->OUVRAGE_SHOW_PRODUCT_QTY == 1) || (isset($_SESSION['OUVRAGE_SHOW_PRODUCT_QTY']) && $_SESSION['OUVRAGE_SHOW_PRODUCT_QTY'] == 1) ? 'checked="checked"' : '';
		$checkedHideAmount = (!isset($_SESSION['OUVRAGE_HIDE_MONTANT']) && $conf->global->OUVRAGE_HIDE_MONTANT == 1) || (isset($_SESSION['OUVRAGE_HIDE_MONTANT']) && $_SESSION['OUVRAGE_HIDE_MONTANT'] == 1) ? 'checked="checked"' : '';

		if (empty($conf->global->OUVRAGE_USE_LINE_VAT)) {
			$out .= '<tr ' . $bc[$var] . '>';
			$out .= '<td colspan="4"><input type="checkbox" name="OUVRAGE_HIDE_PRODUCT_DETAIL" value="1"' . $checkedHideDetails . ' /> ' . $langs->trans('OUVRAGE_HIDE_PRODUCT_DETAIL') . '</td>';
			$out .= '</tr>';
			$out .= '<tr ' . $bc[$var] . '>';
			$out .= '<td colspan="4"><input type="checkbox" name="OUVRAGE_HIDE_PRODUCT_DESCRIPTION" value="1"' . $checkedHideDesc . ' /> ' . $langs->trans('OUVRAGE_HIDE_PRODUCT_DESCRIPTION') . '</td>';
			$out .= '</tr>';
            $out .= '<tr ' . $bc[$var] . '>';
            $out .= '<td colspan="4"><input type="checkbox" name="OUVRAGE_SHOW_PRODUCT_QTY" value="1"' . $checkedShowProductQty . ' /> ' . $langs->trans('OUVRAGE_SHOW_PRODUCT_QTY') . '</td>';
            $out .= '</tr>';
		}
		$out .= '<tr ' . $bc[$var] . '>';
		$out .= '<td colspan="4"><input type="checkbox" name="OUVRAGE_HIDE_MONTANT" value="1"' . $checkedHideAmount . ' /> ' . $langs->trans('OUVRAGE_HIDE_MONTANT') . '</td>';
		$out .= '</tr>';

		$out .= <<<HEREDOC
                        <script>
                        $('input[type=checkbox][name=OUVRAGE_HIDE_PRODUCT_DETAIL]').on('change', function(){
                            if ($(this).prop('checked')) {
                                $('input[type=checkbox][name=OUVRAGE_HIDE_PRODUCT_DESCRIPTION]').prop('checked', true);
                                $('input[type=checkbox][name=OUVRAGE_SHOW_PRODUCT_QTY]').prop('checked', false);
                                $('input[type=checkbox][name=OUVRAGE_HIDE_MONTANT]').prop('checked', false);
                            }
                        });
                        $('input[type=checkbox][name=OUVRAGE_HIDE_PRODUCT_DESCRIPTION]').on('change', function(){
                            if ($(this).prop('checked')) {
                                $('input[type=checkbox][name=OUVRAGE_HIDE_MONTANT]').prop('checked', false);
                                $('input[type=checkbox][name=OUVRAGE_SHOW_PRODUCT_QTY]').prop('checked', false);
                            }
                        });
                        $('input[type=checkbox][name=OUVRAGE_HIDE_MONTANT]').on('change', function(){
                            if ($(this).prop('checked')) {
                                $('input[type=checkbox][name=OUVRAGE_HIDE_PRODUCT_DETAIL]').prop('checked', false);
                                $('input[type=checkbox][name=OUVRAGE_HIDE_PRODUCT_DESCRIPTION]').prop('checked', false);
                                $('input[type=checkbox][name=OUVRAGE_SHOW_PRODUCT_QTY]').prop('checked', true);
                            }
                        });
                        </script>

HEREDOC;

		$this->resprints = $out;
		return 0;
	}


	/**
	 *    Return line description translated in outputlangs and encoded in UTF8
	 *
	 * @param array  $parameters Extra parameters
	 * @param object $object     Object
	 * @param string $action     Type of action
	 *
	 * @return        void
	 */
	function pdf_writelinedesc($parameters = false, &$object, &$action = '')
	{
		global $conf;

		if (!empty($conf->global->OUVRAGE_USE_LINE_VAT)) {
			$_POST['OUVRAGE_HIDE_PRODUCT_DETAIL'] = 0;
		}

		$conf->global->MAIN_PDF_DASH_BETWEEN_LINES = $this->dash;

		if (is_array($parameters) && !empty($parameters)) {
			foreach ($parameters as $key => $value) {
				$$key = $value;
			}
		}
		if (preg_match('/^InfraSPlus_/', $object->modelpdf)) {
			return 0;
		}    // InfraS add

		$backgroundcolor = array('230', '230', '230');
		$pdf->SetFillColor($backgroundcolor[0], $backgroundcolor[1], $backgroundcolor[2]);
		$pdf->SetXY($posx, $posy);

		$return = 0;

		if (Ouvrage::isOuvrage($object->lines[$i])) {    // InfraS change
			$object->lines[$i]->rowid = (!empty($object->lines[$i]->rowid) ? $object->lines[$i]->rowid : $object->lines[$i]->id);

			$pdf->SetFillColor($backgroundcolor[0], $backgroundcolor[1], $backgroundcolor[2]);
			$pdf->MultiCell($pdf->page_largeur - $posx - $pdf->marge_droite, 5, '', 0, '', 1);
			$pdf->SetFont('', 'B', 9);
			$pdf->writeHTMLCell($w, $h, $posx, $posy , $outputlangs->convToOutputCharset($object->lines[$i]->label), 0, 1);

			if (GETPOST('OUVRAGE_HIDE_PRODUCT_DETAIL') == 1) {
				if ($object->lines[$i]->product_type == 9) {
					$pdf->SetFont('', '', 9);
					$pdf->writeHTMLCell($w + 4, $h, $posx, $posy + 6,$outputlangs->convToOutputCharset($object->lines[$i]->desc), 0, 1, false, true, 'J', true);
					return 1;
				} else {
					return 1;
				}
			} else {
				$Y = $pdf->GetY();
				$description = dol_htmlentitiesbr($object->lines[$i]->desc, 1);
				if (!empty($description)) {
					$pdf->SetFont('', '', 9);
					$pdf->writeHTMLCell($w + 4, $h, $posx, $Y + 2, $outputlangs->convToOutputCharset($description), 0, 1);
				}
			}

			$pdf->SetFont('', '', 9);

			$conf->global->MAIN_PDF_DASH_BETWEEN_LINES = 0;

			$return++;
		} elseif (isset($object->lines[$i]->fk_parent_line)) {
			$Y = $pdf->GetY();
			// InfraS change begin
			// Vérifie que la ligne appartient à un ouvrage
			$inOuvrage = false;
			foreach ($object->lines as $k => $v) {
				if ($object->lines[$i]->fk_parent_line == $object->lines[$k]->rowid && Ouvrage::isOuvrage($object->lines[$k])) {
					$inOuvrage = true;
				}
			}

			// InfraS change end
			if ($inOuvrage) {
				if (GETPOST('OUVRAGE_HIDE_PRODUCT_DETAIL') == 1) {
					$conf->global->MAIN_PDF_DASH_BETWEEN_LINES = $this->dash;
					return 1;
				}

				if (empty($object->lines[$i]->product_label)) {
					$text = dol_htmlentitiesbr($object->lines[$i]->desc, 1);
				} else {
					$text = $object->lines[$i]->product_label;
					if (!empty($object->lines[$i]->desc)) {
						$pdf->SetFont('', '', 9);
						$text .= "<br />" . $object->lines[$i]->desc;
					}
				}

				$pdf->writeHTMLCell($w + 4, $h, $posx, $Y, ' - ' . $outputlangs->convToOutputCharset($text), 0, 1);

				$return++;
			}
		} elseif (GETPOST('OUVRAGE_HIDE_PRODUCT_DETAIL') == 1) {
			$conf->global->MAIN_PDF_DASH_BETWEEN_LINES = $this->dash;
			return 0;
		}

		// $nexy = $pdf->GetY();
		//$pdf->SetXY($posx, $nexy);

		return $return;
	}

	/**
	 *    Return line vat rate
	 *
	 * @param object                Object
	 * @param        $i                    Current line number
	 * @param outputlang            Object lang for output0
	 */
	function pdf_getlinevatrate($parameters = false, $object, $action = '')
	{
		global $conf;

		if (!empty($conf->global->OUVRAGE_USE_LINE_VAT)) {
			$_POST['OUVRAGE_HIDE_PRODUCT_DETAIL'] = 0;
			$_POST['OUVRAGE_HIDE_PRODUCT_DESCRIPTION'] = 0;
		}

		if (is_array($parameters) && !empty($parameters)) {
			foreach ($parameters as $key => $value) {
				$$key = $value;
			}
		}
		if (preg_match('/^InfraSPlus_/', $object->modelpdf)) {
			return 0;
		}    // InfraS add

		// si c'est un ouvrage

		if (Ouvrage::isOuvrage($object->lines[$i])) {    // InfraS change
			if ($conf->global->OUVRAGE_USE_LINE_VAT == 1 || GETPOST('OUVRAGE_HIDE_MONTANT') == 1) {
				$out = '';
			} else {
				$out = vatrate($object->lines[$i]->tva_tx, 1, $object->lines[$i]->info_bits);
			}
		} elseif (isset($object->lines[$i]->fk_parent_line)) {
			// InfraS change begin
			// Vérifie que la ligne appartient à un ouvrage
			$inOuvrage = false;
			foreach ($object->lines as $k => $v) {
				if ($object->lines[$i]->fk_parent_line == $object->lines[$k]->rowid && Ouvrage::isOuvrage($object->lines[$k])) {
					$inOuvrage = true;
				}
			}
			// InfraS change end
			if ($inOuvrage) {
				if ((GETPOST('OUVRAGE_HIDE_PRODUCT_DETAIL') == 1 || GETPOST('OUVRAGE_HIDE_PRODUCT_DESCRIPTION') == 1) && GETPOST('OUVRAGE_HIDE_MONTANT') == 1) {
					$out = '';
				} elseif ((GETPOST('OUVRAGE_HIDE_PRODUCT_DETAIL') == 1 || GETPOST('OUVRAGE_HIDE_PRODUCT_DESCRIPTION') == 1) && $object->lines[$i]->product_type != 9) {
					$out = '';
				} elseif (GETPOST('OUVRAGE_HIDE_MONTANT') == 1 && $object->lines[$i]->product_type == 9 && $object->lines[$i]->special_code == $this->special_code) {
					$out = '';
				} else {
					$out = vatrate($object->lines[$i]->tva_tx, 1, $object->lines[$i]->info_bits);
				}
			}
		} else {
			return 0;
		}

		$this->resprints = $out;
		return 1;
	}

	function pdf_getlineupexcltax($parameters = false, $object, $action = '')
	{
		if (is_array($parameters) && !empty($parameters)) {
			foreach ($parameters as $key => $value) {
				$$key = $value;
			}
		}
		if (preg_match('/^InfraSPlus_/', $object->modelpdf)) {
			return 0;
		}    // InfraS add

		$out = '';

		// si c'est un ouvrage

		if (Ouvrage::isOuvrage($object->lines[$i])) {    // InfraS change
			if (GETPOST('OUVRAGE_HIDE_MONTANT') == 1) {
				$out = '';
			} else {
				$out = price($object->lines[$i]->subprice);
			}
		} elseif (isset($object->lines[$i]->fk_parent_line)) {
			// InfraS change begin
			// Vérifie que la ligne appartient à un ouvrage
			$inOuvrage = false;
			foreach ($object->lines as $k => $v) {
				if ($object->lines[$i]->fk_parent_line == $object->lines[$k]->rowid && Ouvrage::isOuvrage($object->lines[$k])) {
					$inOuvrage = true;
				}
			}
			// InfraS change end
			if ($inOuvrage) {
				if ((GETPOST('OUVRAGE_HIDE_PRODUCT_DETAIL') == 1 || GETPOST('OUVRAGE_HIDE_PRODUCT_DESCRIPTION') == 1) && GETPOST('OUVRAGE_HIDE_MONTANT') == 1) {
					$out = '';
				} elseif ((GETPOST('OUVRAGE_HIDE_PRODUCT_DETAIL') == 1 || GETPOST('OUVRAGE_HIDE_PRODUCT_DESCRIPTION') == 1) && $object->lines[$i]->product_type != 9) {
					$out = '';
				} elseif (GETPOST('OUVRAGE_HIDE_MONTANT') == 1 && $object->lines[$i]->product_type == 9 && $object->lines[$i]->special_code == $this->special_code) {
					$out = '';
				} else {
					$out = price($object->lines[$i]->subprice);
				}
			}
		} else {
			return 0;
		}

		$this->resprints = $out;
		return 1;
	}

	function pdf_getlineqty($parameters = false, $object, $action = '')
	{
		if (!empty($conf->global->OUVRAGE_USE_LINE_VAT)) {
			$_POST['OUVRAGE_HIDE_PRODUCT_DETAIL'] = 0;
			$_POST['OUVRAGE_HIDE_PRODUCT_DESCRIPTION'] = 0;
		}

		if (is_array($parameters) && !empty($parameters)) {
			foreach ($parameters as $key => $value) {
				$$key = $value;
			}
		}

		// si c'est un ouvrage

		if (Ouvrage::isOuvrage($object->lines[$i])) {    // InfraS change
			if (GETPOST('OUVRAGE_HIDE_MONTANT') == 1) {
				$out = '';
			} else {
				$out = $object->lines[$i]->qty;
			}
		} elseif (isset($object->lines[$i]->fk_parent_line)) {
			// InfraS change begin
			// Vérifie que la ligne appartient à un ouvrage
			$inOuvrage = false;
			foreach ($object->lines as $k => $v) {
				if ($object->lines[$i]->fk_parent_line == $object->lines[$k]->rowid && Ouvrage::isOuvrage($object->lines[$k])) {
					$inOuvrage = true;
				}
			}
			// InfraS change end

			if ($inOuvrage) {
				if ((GETPOST('OUVRAGE_HIDE_PRODUCT_DETAIL') == 1 || GETPOST('OUVRAGE_HIDE_PRODUCT_DESCRIPTION') == 1) && GETPOST('OUVRAGE_HIDE_MONTANT') == 1) {
					$out = '';
				} elseif ((GETPOST('OUVRAGE_HIDE_PRODUCT_DETAIL') == 1 || GETPOST('OUVRAGE_HIDE_PRODUCT_DESCRIPTION') == 1) && GETPOST('OUVRAGE_SHOW_PRODUCT_QTY', 'int') != 1 && $object->lines[$i]->product_type != 9) {
					$out = '';
				} elseif (GETPOST('OUVRAGE_HIDE_MONTANT') == 1 && $object->lines[$i]->product_type == 9 && $object->lines[$i]->special_code == $this->special_code) {
					$out = '';
				} else {
                    if (GETPOST('OUVRAGE_SHOW_PRODUCT_QTY', 'int') == 1) {

                        $out = $object->lines[$i]->qty;
                    }
				}
			}
		} else {
			return 0;
		}

		$this->resprints = $out;
		return 1;
	}

	/**
	 *    Return line remise percent
	 *
	 * @param object                Object
	 * @param        $i                    Current line number
	 * @param outputlang            Object lang for output
	 */
	function pdf_getlineremisepercent($parameters = false, $object, $action = '')
	{
		if (!empty($conf->global->OUVRAGE_USE_LINE_VAT)) {
			$_POST['OUVRAGE_HIDE_PRODUCT_DETAIL'] = 0;
			$_POST['OUVRAGE_HIDE_PRODUCT_DESCRIPTION'] = 0;
		}

		if (is_array($parameters) && !empty($parameters)) {
			foreach ($parameters as $key => $value) {
				$$key = $value;
			}
		}
		if (preg_match('/^InfraSPlus_/', $object->modelpdf)) {
			return 0;
		}    // InfraS add

		$out = '';

		// si c'est un ouvrage

		if (Ouvrage::isOuvrage($object->lines[$i])) {    // InfraS change
			if (GETPOST('OUVRAGE_HIDE_MONTANT') == 1) {
				$out = '';
			} else {
				$out = dol_print_reduction($object->lines[$i]->remise_percent, $outputlangs);
			}
		} elseif (isset($object->lines[$i]->fk_parent_line)) {
			// InfraS change begin
			// Vérifie que la ligne appartient à un ouvrage
			$inOuvrage = false;
			foreach ($object->lines as $k => $v) {
				if ($object->lines[$i]->fk_parent_line == $object->lines[$k]->rowid && Ouvrage::isOuvrage($object->lines[$k])) {
					$inOuvrage = true;
				}
			}
			// InfraS change end
			if ($inOuvrage) {
				if ((GETPOST('OUVRAGE_HIDE_PRODUCT_DETAIL') == 1 || GETPOST('OUVRAGE_HIDE_PRODUCT_DESCRIPTION') == 1) && GETPOST('OUVRAGE_HIDE_MONTANT') == 1) {
					$out = '';
				} elseif ((GETPOST('OUVRAGE_HIDE_PRODUCT_DETAIL') == 1 || GETPOST('OUVRAGE_HIDE_PRODUCT_DESCRIPTION') == 1) && $object->lines[$i]->product_type != 9) {
					$out = '';
				} elseif (GETPOST('OUVRAGE_HIDE_MONTANT') == 1 && $object->lines[$i]->product_type == 9 && $object->lines[$i]->special_code == $this->special_code) {
					$out = '';
				} else {
					$out = dol_print_reduction($object->lines[$i]->remise_percent, $outputlangs);
				}
			}
		} else {
			return 0;
		}

		$this->resprints = $out;
		return 1;
	}

	/**
	 *    Return line total including tax
	 *
	 * @param object                Object
	 * @param        $i                    Current line number
	 * @param outputlang            Object lang for output
	 */
	function pdf_getlinetotalexcltax($parameters = false, $object, $action = '')
	{
		if (!empty($conf->global->OUVRAGE_USE_LINE_VAT)) {
			$_POST['OUVRAGE_HIDE_PRODUCT_DETAIL'] = 0;
			$_POST['OUVRAGE_HIDE_PRODUCT_DESCRIPTION'] = 0;
		}

		if (is_array($parameters) && !empty($parameters)) {
			foreach ($parameters as $key => $value) {
				$$key = $value;
			}
		}
		if (preg_match('/^InfraSPlus_/', $object->modelpdf)) {
			return 0;
		}    // InfraS add

		// si c'est un ouvrage

		if (Ouvrage::isOuvrage($object->lines[$i])) {    // InfraS change
			if (GETPOST('OUVRAGE_HIDE_MONTANT') == 1) {
				$out = '';
			} else {
				$out = price($object->lines[$i]->total_ht);
			}
		} elseif (isset($object->lines[$i]->fk_parent_line)) {
			// InfraS change begin
			// Vérifie que la ligne appartient à un ouvrage
			$inOuvrage = false;
			foreach ($object->lines as $k => $v) {
				if ($object->lines[$i]->fk_parent_line == $object->lines[$k]->rowid && Ouvrage::isOuvrage($object->lines[$k])) {
					$inOuvrage = true;
				}
			}
			// InfraS change end
			if ($inOuvrage) {
				if ((GETPOST('OUVRAGE_HIDE_PRODUCT_DETAIL') == 1 || GETPOST('OUVRAGE_HIDE_PRODUCT_DESCRIPTION') == 1) && GETPOST('OUVRAGE_HIDE_MONTANT') == 1) {
					$out = '';
				} elseif ((GETPOST('OUVRAGE_HIDE_PRODUCT_DETAIL') == 1 || GETPOST('OUVRAGE_HIDE_PRODUCT_DESCRIPTION') == 1) && $object->lines[$i]->product_type != 9) {
					$out = '';
				} elseif (GETPOST('OUVRAGE_HIDE_MONTANT') == 1 && $object->lines[$i]->product_type == 9 && $object->lines[$i]->special_code == $this->special_code) {
					$out = '';
				} else {
					$out = price($object->lines[$i]->total_ht);
				}
			}
		} else {
			return 0;
		}

		$this->resprints = $out;
		return 1;
	}

	/**
	 *    Return line unit
	 *
	 * @param Object      $object      Object
	 * @param int         $i           Current line number
	 * @param Translate   $outputlangs Object langs for output
	 * @param int         $hidedetails Hide details (0=no, 1=yes, 2=just special lines)
	 * @param HookManager $hookmanager Hook manager instance
	 *
	 * @return    string                            Value for unit cell
	 */
	function pdf_getlineunit($parameters = false, $object, $action = '')
	{
		global $langs;

		if (!empty($conf->global->OUVRAGE_USE_LINE_VAT)) {
			$_POST['OUVRAGE_HIDE_PRODUCT_DETAIL'] = 0;
			$_POST['OUVRAGE_HIDE_PRODUCT_DESCRIPTION'] = 0;
		}

		$i = $parameters['i'];

		if (is_array($parameters) && !empty($parameters)) {
			foreach ($parameters as $key => $value) {
				$$key = $value;
			}
		}

		// si c'est un ouvrage

		if (Ouvrage::isOuvrage($object->lines[$i])) {    // InfraS change
			if (GETPOST('OUVRAGE_HIDE_MONTANT') == 1) {
				$out = '';
			} else {
				$out = $langs->transnoentitiesnoconv($object->lines[$i]->getLabelOfUnit('short'));
			}
		} elseif (isset($object->lines[$i]->fk_parent_line)) {
			// InfraS change begin
			// Vérifie que la ligne appartient à un ouvrage
			$inOuvrage = false;
			foreach ($object->lines as $k => $v) {
				if ($object->lines[$i]->fk_parent_line == $object->lines[$k]->rowid && Ouvrage::isOuvrage($object->lines[$k])) {
					$inOuvrage = true;
				}
			}
			// InfraS change end
			if ($inOuvrage) {
				if ((GETPOST('OUVRAGE_HIDE_PRODUCT_DETAIL') == 1 || GETPOST('OUVRAGE_HIDE_PRODUCT_DESCRIPTION') == 1) && GETPOST('OUVRAGE_HIDE_MONTANT') == 1) {
					$out = '';
				} elseif ((GETPOST('OUVRAGE_HIDE_PRODUCT_DETAIL') == 1 || GETPOST('OUVRAGE_HIDE_PRODUCT_DESCRIPTION') == 1) && $object->lines[$i]->product_type != 9) {
					$out = '';
				} elseif (GETPOST('OUVRAGE_HIDE_MONTANT') == 1 && $object->lines[$i]->product_type == 9 && $object->lines[$i]->special_code == $this->special_code) {
					$out = '';
				} else {
					$out = $langs->transnoentitiesnoconv($object->lines[$i]->getLabelOfUnit('short'));
				}
			}
		} else {
			return 0;
		}

		$this->resprints = $out;
		return 1;
	}

	/**
	 * Return line percent
	 *
	 * @param Object      $object      Object
	 * @param int         $i           Current line number
	 * @param Translate   $outputlangs Object langs for output
	 * @param int         $hidedetails Hide details (0=no, 1=yes, 2=just special lines)
	 * @param HookManager $hookmanager Hook manager instance
	 *
	 * @return string
	 */
	function pdf_getlineprogress($parameters = false, $object, $action = '')
	{
		global $langs, $conf;

		if (!empty($conf->global->OUVRAGE_USE_LINE_VAT)) {
			$_POST['OUVRAGE_HIDE_PRODUCT_DETAIL'] = 0;
			$_POST['OUVRAGE_HIDE_PRODUCT_DESCRIPTION'] = 0;
		}

		if (is_array($parameters) && !empty($parameters)) {
			foreach ($parameters as $key => $value) {
				$$key = $value;
			}
		}
		if (preg_match('/^InfraSPlus_/', $object->modelpdf)) {
			return 0;
		}    // InfraS add

		// si c'est un ouvrage

		if (Ouvrage::isOuvrage($object->lines[$i])) {    // InfraS change
			if (GETPOST('OUVRAGE_HIDE_MONTANT') == 1) {
				$out = '';
			} else {
				if ($conf->global->SITUATION_DISPLAY_DIFF_ON_PDF) {
					$prev_progress = 0;
					if (method_exists($object, 'get_prev_progress')) {
						$prev_progress = $object->lines[$i]->get_prev_progress($object->id);
					}
					$out = ($object->lines[$i]->situation_percent - $prev_progress) . '%';
				} else {
					$out = $object->lines[$i]->situation_percent . '%';
				}
			}
		} elseif (isset($object->lines[$i]->fk_parent_line)) {
			// InfraS change begin
			// Vérifie que la ligne appartient à un ouvrage
			$inOuvrage = false;
			foreach ($object->lines as $k => $v) {
				if ($object->lines[$i]->fk_parent_line == $object->lines[$k]->rowid && Ouvrage::isOuvrage($object->lines[$k])) {
					$inOuvrage = true;
				}
			}
			// InfraS change end
			if ($inOuvrage) {
				if ((GETPOST('OUVRAGE_HIDE_PRODUCT_DETAIL') == 1 || GETPOST('OUVRAGE_HIDE_PRODUCT_DESCRIPTION') == 1) && GETPOST('OUVRAGE_HIDE_MONTANT') == 1) {
					$out = '';
				} elseif ((GETPOST('OUVRAGE_HIDE_PRODUCT_DETAIL') == 1 || GETPOST('OUVRAGE_HIDE_PRODUCT_DESCRIPTION') == 1) && $object->lines[$i]->product_type != 9) {
					$out = '';
				} elseif (GETPOST('OUVRAGE_HIDE_MONTANT') == 1 && $object->lines[$i]->product_type == 9 && $object->lines[$i]->special_code == $this->special_code) {
					$out = '';
				} else {
					if ($conf->global->SITUATION_DISPLAY_DIFF_ON_PDF) {
						$prev_progress = 0;
						if (method_exists($object, 'get_prev_progress')) {
							$prev_progress = $object->lines[$i]->get_prev_progress($object->id);
						}
						$out = ($object->lines[$i]->situation_percent - $prev_progress) . '%';
					} else {
						$out = $object->lines[$i]->situation_percent . '%';
					}
				}
			}
		} else {
			return 0;
		}

		$this->resprints = $out;
		return 1;
	}
}
